// AnimaGate AI Suite - バニラJSでの実装
document.addEventListener('DOMContentLoaded', function() {
    // フォーム要素の取得
    const form = document.getElementById('generate-content-form');
    if (!form) return;
    
    // スライダーの値表示更新
    const temperatureSlider = document.getElementById('temperature');
    if (temperatureSlider) {
        temperatureSlider.addEventListener('input', function() {
            const temperatureValue = document.getElementById('temperature-value');
            if (temperatureValue) {
                temperatureValue.textContent = this.value;
            }
        });
    }

    // 通知メッセージコンテナの作成（存在しない場合）
    let resultContainer = document.getElementById('generation-result-container');
    if (!resultContainer) {
        resultContainer = document.createElement('div');
        resultContainer.id = 'generation-result-container';
        resultContainer.setAttribute('aria-live', 'polite');
        // フォームの後に挿入
        form.parentNode.insertBefore(resultContainer, form.nextSibling);
    }

    // フォーム送信処理
    form.addEventListener('submit', function(e) {
        e.preventDefault();
        
        const button = document.getElementById('generate-content-button');
        const status = document.getElementById('generation-status');
        
        // フォームデータ取得
        const title = document.getElementById('content_title').value;
        const keywords = document.getElementById('content_keywords').value;
        const type = document.getElementById('content_type').value;
        const length = document.getElementById('content_length').value;
        const tone = document.getElementById('content_tone').value;
        const additionalInstructions = document.getElementById('additional_instructions').value;
        
        // オプション
        const autoHeadings = document.getElementById('auto_headings').checked ? 1 : 0;
        const addConclusion = document.getElementById('add_conclusion').checked ? 1 : 0;
        const includeFaq = document.getElementById('include_faq').checked ? 1 : 0;
        
        // 入力検証
        if (!title) {
            alert(animagate_ai_suite.empty_title);
            return;
        }
        
        // ボタンと状態の更新
        button.disabled = true;
        if (status) {
            status.classList.remove('hidden');
            status.textContent = animagate_ai_suite.generating;
        }
        
        // 結果エリアをリセット
        clearElement(resultContainer);
        
        // フォームデータの作成
        const formData = new FormData();
        formData.append('action', 'generate_ai_content');
        formData.append('nonce', animagate_ai_suite.nonce);
        formData.append('title', title);
        formData.append('keywords', keywords);
        formData.append('type', type);
        formData.append('length', length);
        formData.append('tone', tone);
        formData.append('additional_instructions', additionalInstructions);
        formData.append('auto_headings', autoHeadings);
        formData.append('add_conclusion', addConclusion);
        formData.append('include_faq', includeFaq);
        
        // Ajax リクエスト
        fetch(animagate_ai_suite.ajax_url, {
            method: 'POST',
            body: formData,
            credentials: 'same-origin'
        })
        .then(response => response.json())
        .then(data => {
            // ここにコンソールログを追加
            console.log('サーバーレスポンス:', data);

            button.disabled = false;
            if (status) {
                status.classList.add('hidden');
            }
            
            if (data.success) {
                // 安全な通知作成 - innerHTML を使わない
                createSuccessNotice(
                    resultContainer, 
                    data.data.message, 
                    data.data.edit_url, 
                    animagate_ai_suite.view_post || '記事を編集'
                );
                
                // スクロールして通知を表示
                resultContainer.scrollIntoView({ behavior: 'smooth' });
            } else {
                alert(data.data.message || animagate_ai_suite.error);
            }
        })
        .catch(error => {
            // ここにエラー詳細ログを追加
            console.error('エラー詳細:', error);

            button.disabled = false;
            if (status) {
                status.classList.add('hidden');
            }
            alert(animagate_ai_suite.error);
        });
    });
    
    // スタイルシートを追加
    const style = document.createElement('style');
    style.textContent = `
        /* WordPress標準通知の閉じるボタン用スタイル */
        .notice-dismiss {
            position: absolute;
            top: 0;
            right: 1px;
            border: none;
            margin: 0;
            padding: 9px;
            background: none;
            color: #72777c;
            cursor: pointer;
        }
        .notice-dismiss:hover {
            color: #c00;
        }
        .notice-dismiss:before {
            content: "\\f153";
            font: normal 16px/20px dashicons;
        }
    `;
    document.head.appendChild(style);
});

// 要素の中身をクリア
function clearElement(element) {
    if (!element) return;
    while (element.firstChild) {
        element.removeChild(element.firstChild);
    }
}

// XSS 対策としてエスケープ関数を強化
function escapeHTML(str) {
    if (typeof str !== 'string') {
        return '';
    }
    
    const div = document.createElement('div');
    div.textContent = str; // textContentはHTMLエスケープを行う
    return div.textContent;
}

// URL検証関数を追加
function isValidUrl(url) {
    try {
        const parsedUrl = new URL(url);
        // 許可するプロトコルのみ許可
        return ['http:', 'https:'].includes(parsedUrl.protocol);
    } catch (e) {
        return false;
    }
}

// DOM を使った安全な通知作成関数
function createSuccessNotice(container, message, editUrl, viewPostText) {
    // コンテナをクリア
    clearElement(container);
    
    // 通知要素を作成
    const notice = document.createElement('div');
    notice.className = 'notice notice-success is-dismissible';
    
    // メッセージ段落
    const messageParagraph = document.createElement('p');
    messageParagraph.textContent = message; // textContent は自動的にエスケープされる
    notice.appendChild(messageParagraph);
    
    // 編集リンク段落
    const linkParagraph = document.createElement('p');
    const editLink = document.createElement('a');
    
    // URL の検証
    if (isValidUrl(editUrl)) {
        editLink.href = editUrl;
    } else {
        editLink.href = '#'; // 無効な URL の場合
        console.error('Invalid URL detected:', editUrl);
    }
    
    editLink.className = 'button button-primary';
    editLink.textContent = viewPostText || '記事を編集';
    linkParagraph.appendChild(editLink);
    notice.appendChild(linkParagraph);
    
    // 閉じるボタンの追加
    const dismissButton = document.createElement('button');
    dismissButton.type = 'button';
    dismissButton.className = 'notice-dismiss';
    dismissButton.addEventListener('click', function() {
        notice.remove();
    });
    notice.appendChild(dismissButton);
    
    // 通知を追加
    container.appendChild(notice);
}