<?php
/**
 * 管理画面関連の機能を実装するクラス
 */
class AnimaGate_AI_Suite_Admin {
    private $plugin_options;

    public function __construct($plugin_options) {
        $this->plugin_options = $plugin_options;
    }

    /**
     * 初期化
     */
    public function init() {
        // 管理画面メニューを追加
        add_action('admin_menu', array($this, 'add_admin_menu'));
        
        // 設定の登録
        add_action('admin_init', array($this, 'register_settings'));
        
        // スクリプトとスタイルの追加
        add_action('admin_enqueue_scripts', array($this, 'enqueue_scripts'));
    }

    /**
     * 管理画面メニューを追加
     */
    public function add_admin_menu() {
        add_menu_page(
            __('AnimaGate AI Suite', AGAS_TEXT_DOMAIN),
            __('AnimaGate AI Suite', AGAS_TEXT_DOMAIN),
            'edit_posts',
            'animagate-ai-suite',
            array($this, 'render_generation_page'),
            'dashicons-editor-paste-text',
            30
        );
        
        add_submenu_page(
            'animagate-ai-suite',
            __('コンテンツ生成', AGAS_TEXT_DOMAIN),
            __('コンテンツ生成', AGAS_TEXT_DOMAIN),
            'edit_posts',
            'animagate-ai-suite',
            array($this, 'render_generation_page')
        );
        
        add_submenu_page(
            'animagate-ai-suite',
            __('設定', AGAS_TEXT_DOMAIN),
            __('設定', AGAS_TEXT_DOMAIN),
            'manage_options',
            'animagate-ai-suite-settings',
            array($this, 'render_settings_page')
        );
    }

    /**
     * 設定を登録
     */
    public function register_settings() {
        register_setting(
            'animagate_ai_suite_options_group',
            'animagate_ai_suite_options',
            array($this, 'sanitize_options')
        );

        add_settings_section(
            'animagate_ai_suite_main',
            __('API設定', AGAS_TEXT_DOMAIN),
            array($this, 'settings_section_callback'),
            'animagate-ai-suite-settings'
        );

        add_settings_field(
            'api_key',
            __('OpenAI APIキー', AGAS_TEXT_DOMAIN),
            array($this, 'api_key_callback'),
            'animagate-ai-suite-settings',
            'animagate_ai_suite_main'
        );

        add_settings_field(
            'model',
            __('使用するモデル', AGAS_TEXT_DOMAIN),
            array($this, 'model_callback'),
            'animagate-ai-suite-settings',
            'animagate_ai_suite_main'
        );

        add_settings_field(
            'max_tokens',
            __('最大トークン数', AGAS_TEXT_DOMAIN),
            array($this, 'max_tokens_callback'),
            'animagate-ai-suite-settings',
            'animagate_ai_suite_main'
        );

        add_settings_field(
            'temperature',
            __('温度（創造性）', AGAS_TEXT_DOMAIN),
            array($this, 'temperature_callback'),
            'animagate-ai-suite-settings',
            'animagate_ai_suite_main'
        );
    }

    /**
     * オプションのサニタイズ
     */
    public function sanitize_options($input) {
        $new_input = array();
        
        if (isset($_POST['animagate_ai_suite_settings_nonce']) && wp_verify_nonce($_POST['animagate_ai_suite_settings_nonce'], 'animagate_ai_suite_settings_nonce')) {
            // APIキーのサニタイズ
            if (isset($input['api_key'])) {
                $new_input['api_key'] = sanitize_text_field($input['api_key']);
            }
            
            // モデルのサニタイズと検証
            if (isset($input['model'])) {
                $allowed_models = array('gpt-3.5-turbo', 'gpt-4', 'gpt-4-turbo');
                $model = sanitize_text_field($input['model']);
                $new_input['model'] = in_array($model, $allowed_models) ? $model : 'gpt-3.5-turbo';
            }
            
            // 最大トークン数のサニタイズと検証
            if (isset($input['max_tokens'])) {
                $max_tokens = absint($input['max_tokens']);
                $new_input['max_tokens'] = ($max_tokens >= 100 && $max_tokens <= 4000) ? $max_tokens : 1500;
            }
            
            // 温度のサニタイズと検証
            if (isset($input['temperature'])) {
                $temperature = floatval($input['temperature']);
                if ($temperature < 0) {
                    $temperature = 0;
                } elseif ($temperature > 1) {
                    $temperature = 1;
                }
                $new_input['temperature'] = $temperature;
            }
        } else {
            // ノンスが無効な場合、既存の設定を保持
            $new_input = get_option('animagate_ai_suite_options', array());
            add_settings_error(
                'animagate_ai_suite_options',
                'invalid_nonce',
                __('セキュリティチェックに失敗しました。', AGAS_TEXT_DOMAIN)
            );
        }
        
        return $new_input;
    }

    /**
     * 設定セクションのコールバック
     */
    public function settings_section_callback() {
        echo '<p>' . esc_html__('OpenAI APIの設定を行います。APIキーは必須です。', AGAS_TEXT_DOMAIN) . '</p>';
    }

    /**
     * APIキー設定フィールドのコールバック
     */
    public function api_key_callback() {
        $api_key = isset($this->plugin_options['api_key']) ? $this->plugin_options['api_key'] : '';
        
        // APIキーをマスクして表示（機密情報の保護）
        $masked_key = '';
        if (!empty($api_key)) {
            $first_chars = substr($api_key, 0, 4);
            $last_chars = substr($api_key, -4);
            $masked_key = $first_chars . str_repeat('*', strlen($api_key) - 8) . $last_chars;
        }
        
        // type="password" を使用して入力中の値を隠す
        printf(
            '<input type="password" id="api_key" name="animagate_ai_suite_options[api_key]" value="%s" class="regular-text" autocomplete="off" />',
            esc_attr($api_key)
        );
        
        echo '<p class="description">' . esc_html__('OpenAI APIキーを入力してください。', AGAS_TEXT_DOMAIN) . '</p>';
        
        if (!empty($api_key)) {
            echo '<p class="description">' . esc_html__('現在のキー: ', AGAS_TEXT_DOMAIN) . esc_html($masked_key) . '</p>';
        }
    }

    /**
     * モデル設定フィールドのコールバック
     */
    public function model_callback() {
        $model = isset($this->plugin_options['model']) ? $this->plugin_options['model'] : 'gpt-4';
        $models = array(
            'gpt-4' => 'GPT-4',
            'gpt-4-turbo' => 'GPT-4 Turbo',
            'gpt-3.5-turbo' => 'GPT-3.5 Turbo',
        );
        
        echo '<select id="model" name="animagate_ai_suite_options[model]">';
        foreach ($models as $model_id => $model_name) {
            printf(
                '<option value="%s" %s>%s</option>',
                esc_attr($model_id),
                selected($model, $model_id, false),
                esc_html($model_name)
            );
        }
        echo '</select>';
        echo '<p class="description">' . esc_html__('使用するOpenAIのモデルを選択してください。', AGAS_TEXT_DOMAIN) . '</p>';
    }

    /**
     * 最大トークン数設定フィールドのコールバック
     */
    public function max_tokens_callback() {
        $max_tokens = isset($this->plugin_options['max_tokens']) ? $this->plugin_options['max_tokens'] : 1500;
        
        printf(
            '<input type="number" id="max_tokens" name="animagate_ai_suite_options[max_tokens]" value="%s" class="small-text" min="100" max="4000" step="100" />',
            esc_attr($max_tokens)
        );
        echo '<p class="description">' . esc_html__('生成される最大トークン数を設定します。長い記事ほど大きな値が必要です。', AGAS_TEXT_DOMAIN) . '</p>';
    }

    /**
     * 温度設定フィールドのコールバック
     */
    public function temperature_callback() {
        $temperature = isset($this->plugin_options['temperature']) ? $this->plugin_options['temperature'] : 0.7;
        
        printf(
            '<input type="range" id="temperature" name="animagate_ai_suite_options[temperature]" value="%s" min="0" max="1" step="0.1" oninput="document.getElementById(\'temperature-value\').textContent = this.value;" />',
            esc_attr($temperature)
        );
        echo '<span id="temperature-value">' . esc_html($temperature) . '</span>';
        echo '<p class="description">' . esc_html__('値が低いほど予測可能な内容に、高いほど創造的な内容になります。', AGAS_TEXT_DOMAIN) . '</p>';
    }

    /**
     * 設定ページのレンダリング
     */
    public function render_settings_page() {
        if (!current_user_can('manage_options')) {
            return;
        }
        
        ?>
        <div class="wrap">
            <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
            <form action="options.php" method="post">
                <?php
                settings_fields('animagate_ai_suite_options_group');
                do_settings_sections('animagate-ai-suite-settings');
                submit_button();
                ?>
            </form>
        </div>
        <?php
    }

    /**
     * スクリプトとスタイルをエンキュー
     */
    public function enqueue_scripts($hook) {
        if (strpos($hook, 'animagate-ai-suite') === false) {
            return;
        }

        wp_enqueue_style(
            'animagate-ai-suite-style',
            AGAS_PLUGIN_URL . 'assets/css/admin-style.css',
            array(),
            AGAS_VERSION
        );

        wp_enqueue_script(
            'animagate-ai-suite-script',
            AGAS_PLUGIN_URL . 'assets/js/admin-script.js',
            array(),
            AGAS_VERSION,
            true
        );

        wp_localize_script(
            'animagate-ai-suite-script',
            'animagate_ai_suite',
            array(
                'ajax_url' => admin_url('admin-ajax.php'),
                'nonce' => wp_create_nonce('animagate_ai_suite_nonce'),
                'rest_nonce' => wp_create_nonce('wp_rest'), // REST API用のノンスを追加
                'generating' => __('生成中...', AGAS_TEXT_DOMAIN),
                'success' => __('記事が生成されました！', AGAS_TEXT_DOMAIN),
                'error' => __('エラーが発生しました。', AGAS_TEXT_DOMAIN),
                'empty_title' => __('タイトルを入力してください。', AGAS_TEXT_DOMAIN)
            )
        );
    }

    /**
     * 記事生成ページのレンダリング
     */
    public function render_generation_page() {
        $api_key = isset($this->plugin_options['api_key']) ? $this->plugin_options['api_key'] : '';
        
        if (empty($api_key)) {
            $settings_url = admin_url('admin.php?page=animagate-ai-suite-settings');
            ?>
            <div class="wrap">
                <h1><?php echo esc_html__('AI記事生成', AGAS_TEXT_DOMAIN); ?></h1>
                <div class="notice notice-error">
                    <p><?php printf(
                        esc_html__('OpenAI APIキーが設定されていません。%s設定ページ%sでAPIキーを設定してください。', AGAS_TEXT_DOMAIN),
                        '<a href="' . esc_url($settings_url) . '">',
                        '</a>'
                    ); ?></p>
                </div>
            </div>
            <?php
            return;
        }
        
        // 生成フォームのテンプレートをインクルード
        require_once AGAS_PLUGIN_DIR . 'templates/generation-form.php';
    }

    /**
     * 最近生成した記事を表示
     */
    public function render_recent_generated_content() {
        $args = array(
            'post_type' => 'post',
            'post_status' => array('draft', 'publish'),
            'posts_per_page' => 5,
            'orderby' => 'date',
            'order' => 'DESC',
            'meta_query' => array(
                array(
                    'key' => '_ai_generated_content',
                    'value' => '1',
                    'compare' => '='
                )
            )
        );
        
        $recent_content = new WP_Query($args);
        
        if ($recent_content->have_posts()) {
            echo '<table class="wp-list-table widefat fixed striped">';
            echo '<thead><tr>';
            echo '<th>' . esc_html__('タイトル', AGAS_TEXT_DOMAIN) . '</th>';
            echo '<th>' . esc_html__('ステータス', AGAS_TEXT_DOMAIN) . '</th>';
            echo '<th>' . esc_html__('生成日時', AGAS_TEXT_DOMAIN) . '</th>';
            echo '<th>' . esc_html__('操作', AGAS_TEXT_DOMAIN) . '</th>';
            echo '</tr></thead>';
            echo '<tbody>';
            
            while ($recent_content->have_posts()) {
                $recent_content->the_post();
                $post_id = get_the_ID();
                $edit_link = get_edit_post_link($post_id);
                $view_link = get_permalink($post_id);
                $status = get_post_status() === 'publish' ? __('公開', AGAS_TEXT_DOMAIN) : __('下書き', AGAS_TEXT_DOMAIN);
                
                echo '<tr>';
                echo '<td>' . esc_html(get_the_title()) . '</td>';
                echo '<td>' . esc_html($status) . '</td>';
                echo '<td>' . esc_html(get_the_date()) . '</td>';
                echo '<td>';
                
                // リンクを表示する前にURLの安全性を確認
                if (!empty($edit_link)) {
                    echo '<a href="' . esc_url($edit_link) . '" class="button button-small">' . esc_html__('編集', AGAS_TEXT_DOMAIN) . '</a> ';
                }
                
                if (get_post_status() === 'publish' && !empty($view_link)) {
                    echo '<a href="' . esc_url($view_link) . '" class="button button-small" target="_blank" rel="noopener noreferrer">' . esc_html__('表示', AGAS_TEXT_DOMAIN) . '</a>';
                }
                
                echo '</td>';
                echo '</tr>';
            }
            
            echo '</tbody></table>';
        } else {
            echo '<p>' . esc_html__('生成された記事はまだありません。', AGAS_TEXT_DOMAIN) . '</p>';
        }
        
        wp_reset_postdata();
    }
}