<?php
/**
 * ブロックエディター統合機能を実装するクラス
 */

// 直接アクセス禁止
if (!defined('ABSPATH')) {
    exit;
}

class AnimaGate_AI_Suite_Editor {
    private $plugin_options;

    public function __construct($plugin_options) {
        $this->plugin_options = $plugin_options;
        $this->init();
    }

    /**
     * 初期化
     */
    public function init() {
        // エディター拡張を登録
        add_action('enqueue_block_editor_assets', array($this, 'enqueue_editor_assets'));
        
        // REST API エンドポイントを登録
        add_action('rest_api_init', array($this, 'register_rest_routes'));
        
        // セキュリティヘッダーの追加
        add_action('admin_init', array($this, 'add_security_headers'));
    }

    /**
     * セキュリティヘッダーの追加
     */
    public function add_security_headers() {
        // XSS対策
        header('X-XSS-Protection: 1; mode=block');
        
        // クリックジャッキング対策
        header('X-Frame-Options: SAMEORIGIN');
        
        // MIMEタイプスニッフィング対策
        header('X-Content-Type-Options: nosniff');
        
        // HSTS（HTTPSの強制）- 本番環境でのみ有効化
        if (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off') {
            header('Strict-Transport-Security: max-age=31536000; includeSubDomains; preload');
        }
        
        // リファラーポリシー
        header('Referrer-Policy: strict-origin-when-cross-origin');
    }

    /**
     * エディターアセットを読み込み
     */
    public function enqueue_editor_assets() {
        // ビルドされたJavaScriptの読み込み
        wp_enqueue_script(
            'animagate-ai-suite-editor',
            AGAS_PLUGIN_URL . 'build/editor-integration.js',
            array(
                'wp-plugins',
                'wp-edit-post',
                'wp-element',
                'wp-components',
                'wp-i18n',
                'wp-data',
                'wp-blocks',
                'wp-block-editor'
            ),
            AGAS_VERSION,
            true
        );

        // CSSの読み込み
        wp_enqueue_style(
            'animagate-ai-suite-editor-style',
            AGAS_PLUGIN_URL . 'assets/css/editor-style.css',
            array('wp-edit-blocks'),
            AGAS_VERSION
        );

        // JSに渡すデータ
        $localized_data = array(
            'restUrl' => esc_url_raw(rest_url('animagate-ai-suite/v1')),
            'nonce' => wp_create_nonce('wp_rest'),
            'hasApiKey' => !empty($this->plugin_options['api_key']),
            'strings' => array(
                'modalTitle' => esc_html__('AnimaGate AI Suite', AGAS_TEXT_DOMAIN),
                'generateContent' => esc_html__('AnimaGate AI Suite', AGAS_TEXT_DOMAIN),
                'generating' => esc_html__('生成中...', AGAS_TEXT_DOMAIN),
                'success' => esc_html__('生成完了！', AGAS_TEXT_DOMAIN),
                'error' => esc_html__('エラーが発生しました', AGAS_TEXT_DOMAIN),
                'insertContent' => esc_html__('コンテンツを挿入', AGAS_TEXT_DOMAIN),
                'cancel' => esc_html__('キャンセル', AGAS_TEXT_DOMAIN),
                'emptyTitle' => esc_html__('タイトルを入力してください', AGAS_TEXT_DOMAIN),
                'title' => esc_html__('タイトル', AGAS_TEXT_DOMAIN),
                'keywords' => esc_html__('キーワード', AGAS_TEXT_DOMAIN),
                'keywordsDesc' => esc_html__('複数のキーワードはカンマで区切ってください', AGAS_TEXT_DOMAIN),
                'contentType' => esc_html__('記事タイプ', AGAS_TEXT_DOMAIN),
                'contentLength' => esc_html__('記事の長さ', AGAS_TEXT_DOMAIN),
                'contentTone' => esc_html__('トーン', AGAS_TEXT_DOMAIN),
                'additionalInstructions' => esc_html__('追加指示', AGAS_TEXT_DOMAIN),
                'additionalInstructionsDesc' => esc_html__('AIに対する特別な指示があれば入力してください', AGAS_TEXT_DOMAIN),
                'options' => esc_html__('オプション', AGAS_TEXT_DOMAIN),
                'autoHeadings' => esc_html__('見出しを自動生成', AGAS_TEXT_DOMAIN),
                'addConclusion' => esc_html__('まとめ・結論を追加', AGAS_TEXT_DOMAIN),
                'includeFaq' => esc_html__('FAQセクションを追加', AGAS_TEXT_DOMAIN),
                'generate' => esc_html__('生成', AGAS_TEXT_DOMAIN),
                'short' => esc_html__('短い（約500字）', AGAS_TEXT_DOMAIN),
                'medium' => esc_html__('中程度（約1000字）', AGAS_TEXT_DOMAIN),
                'long' => esc_html__('長い（約2000字）', AGAS_TEXT_DOMAIN),
                'blog' => esc_html__('ブログ記事', AGAS_TEXT_DOMAIN),
                'news' => esc_html__('ニュース記事', AGAS_TEXT_DOMAIN),
                'review' => esc_html__('レビュー', AGAS_TEXT_DOMAIN),
                'howto' => esc_html__('ハウツー記事', AGAS_TEXT_DOMAIN),
                'listicle' => esc_html__('リスト記事', AGAS_TEXT_DOMAIN),
                'casual' => esc_html__('カジュアル', AGAS_TEXT_DOMAIN),
                'professional' => esc_html__('プロフェッショナル', AGAS_TEXT_DOMAIN),
                'friendly' => esc_html__('フレンドリー', AGAS_TEXT_DOMAIN),
                'academic' => esc_html__('アカデミック', AGAS_TEXT_DOMAIN),
                'funny' => esc_html__('ユーモア', AGAS_TEXT_DOMAIN),
                'apiKeyMissing' => esc_html__('OpenAI APIキーが設定されていません。設定ページでAPIキーを設定してください。', AGAS_TEXT_DOMAIN),
                'settingsUrl' => esc_url(admin_url('admin.php?page=animagate-ai-suite-settings')),
                'goToSettings' => esc_html__('設定ページへ', AGAS_TEXT_DOMAIN),
                'selectTemplate' => esc_html__('テンプレートを選択', AGAS_TEXT_DOMAIN),
                'noTemplate' => esc_html__('テンプレートなし', AGAS_TEXT_DOMAIN),
                'view_post' => esc_html__('記事を編集', AGAS_TEXT_DOMAIN),
                'excerptGeneration' => esc_html__('抜粋生成', AGAS_TEXT_DOMAIN),
                'contentGeneration' => esc_html__('コンテンツ生成', AGAS_TEXT_DOMAIN),
                'excerptDescription' => esc_html__('記事の内容から抜粋を自動生成します。より魅力的な抜粋文を作成しましょう。', AGAS_TEXT_DOMAIN),
                'generateExcerpt' => esc_html__('抜粋を生成', AGAS_TEXT_DOMAIN),
                'generatedExcerpt' => esc_html__('生成された抜粋', AGAS_TEXT_DOMAIN),
                'insertExcerpt' => esc_html__('抜粋を挿入', AGAS_TEXT_DOMAIN),
                'generateAgain' => esc_html__('再生成', AGAS_TEXT_DOMAIN),
                'noContent' => esc_html__('抜粋を生成するコンテンツがありません。', AGAS_TEXT_DOMAIN),
                'excerptLength' => esc_html__('抜粋の文字数', AGAS_TEXT_DOMAIN),
                'excerptLengthHelp' => esc_html__('50〜500文字の範囲で指定してください（デフォルト: 100文字）', AGAS_TEXT_DOMAIN),
                'excerptShort' => esc_html__('短い（約100文字）', AGAS_TEXT_DOMAIN),
                'excerptNormal' => esc_html__('標準（約200文字）', AGAS_TEXT_DOMAIN),
                'excerptLong' => esc_html__('長い（約300文字）', AGAS_TEXT_DOMAIN),
                'excerptType' => esc_html__('抜粋の長さ', AGAS_TEXT_DOMAIN),
                'excerptTone' => esc_html__('トーン', AGAS_TEXT_DOMAIN),
            ),
        );

        wp_localize_script(
            'animagate-ai-suite-editor',
            'animagateAiSuiteEditor',
            $localized_data
        );

        // テンプレートデータがあれば追加
        $templates_data = $this->get_templates_data();
        if (!empty($templates_data)) {
            $script_data = wp_scripts()->get_data('animagate-ai-suite-editor', 'data');
            if ($script_data) {
                // JSON文字列を適切にサニタイズして埋め込む
                $templates_json = wp_json_encode($templates_data);
                if ($templates_json !== false) {
                    $script_data = str_replace('"templates":[]', '"templates":' . $templates_json, $script_data);
                    wp_scripts()->add_data('animagate-ai-suite-editor', 'data', $script_data);
                }
            }
        }
    }

    /**
     * テンプレートデータを取得
     */
    private function get_templates_data() {
        // テンプレートを取得するためのフィルターを適用
        $templates = apply_filters('animagate_ai_suite_templates', array());
        
        // セキュリティのため、テンプレートデータをサニタイズ
        $sanitized_templates = array();
        foreach ($templates as $template) {
            $sanitized_template = array(
                'id' => isset($template['id']) ? absint($template['id']) : 0,
                'name' => isset($template['name']) ? sanitize_text_field($template['name']) : '',
                'description' => isset($template['description']) ? sanitize_text_field($template['description']) : '',
                'type' => isset($template['type']) ? sanitize_text_field($template['type']) : 'blog',
                'additionalInstructions' => isset($template['additionalInstructions']) ? wp_kses_post($template['additionalInstructions']) : '',
                'autoHeadings' => isset($template['autoHeadings']) ? (bool)$template['autoHeadings'] : true,
                'addConclusion' => isset($template['addConclusion']) ? (bool)$template['addConclusion'] : true,
                'includeFaq' => isset($template['includeFaq']) ? (bool)$template['includeFaq'] : false
            );
            $sanitized_templates[] = $sanitized_template;
        }
        
        return $sanitized_templates;
    }

    /**
     * REST API エンドポイントを登録
     */
    public function register_rest_routes() {
        register_rest_route('animagate-ai-suite/v1', '/generate-content', array(
            'methods' => 'POST',
            'callback' => array($this, 'generate_content_endpoint'),
            'permission_callback' => function() {
                // アクセス権限とノンスの両方をチェック
                return current_user_can('edit_posts');
            },
            'args' => array(
                'title' => array(
                    'required' => true,
                    'type' => 'string',
                    'sanitize_callback' => 'sanitize_text_field',
                    'validate_callback' => function($param) {
                        return !empty($param) && strlen($param) <= 200;
                    }
                ),
                'keywords' => array(
                    'type' => 'string',
                    'sanitize_callback' => 'sanitize_text_field',
                ),
                'type' => array(
                    'type' => 'string',
                    'sanitize_callback' => 'sanitize_text_field',
                    'validate_callback' => function($param) {
                        return in_array($param, array('blog', 'news', 'review', 'howto', 'listicle'));
                    }
                ),
                'length' => array(
                    'type' => 'string',
                    'sanitize_callback' => 'sanitize_text_field',
                    'validate_callback' => function($param) {
                        return in_array($param, array('short', 'medium', 'long'));
                    }
                ),
                'tone' => array(
                    'type' => 'string',
                    'sanitize_callback' => 'sanitize_text_field',
                    'validate_callback' => function($param) {
                        return in_array($param, array('casual', 'professional', 'friendly', 'academic', 'funny'));
                    }
                ),
                'additionalInstructions' => array(
                    'type' => 'string',
                    'sanitize_callback' => 'sanitize_textarea_field',
                ),
                'autoHeadings' => array(
                    'type' => 'boolean',
                ),
                'addConclusion' => array(
                    'type' => 'boolean',
                ),
                'includeFaq' => array(
                    'type' => 'boolean',
                ),
            )
        ));

        register_rest_route('animagate-ai-suite/v1', '/generate-excerpt', array(
            'methods' => 'POST',
            'callback' => array($this, 'generate_excerpt_endpoint'),
            'permission_callback' => function() {
                return current_user_can('edit_posts');
            },
            'args' => array(
                'content' => array(
                    'required' => true,
                    'type' => 'string',
                ),
                'tone' => array(
                    'type' => 'string',
                    'sanitize_callback' => 'sanitize_text_field',
                    'validate_callback' => function($param) {
                        return in_array($param, array('casual', 'professional', 'friendly', 'academic'));
                    }
                ),
                'length' => array(
                    'type' => 'integer',
                    'sanitize_callback' => 'absint',
                    'validate_callback' => function($param) {
                        return $param >= 50 && $param <= 500;
                    }
                ),
            )
        ));

        // 拡張用のエンドポイントを登録できるようにするフック
        do_action('animagate_ai_suite_register_rest_routes');
    }

    /**
     * コンテンツ生成エンドポイント
     */
    public function generate_content_endpoint($request) {
        // ノンスチェック
        $nonce = $request->get_header('X-WP-Nonce');
        if (!wp_verify_nonce($nonce, 'wp_rest')) {
            return new WP_Error(
                'rest_forbidden',
                __('セキュリティチェックに失敗しました。ページを更新してからもう一度お試しください。', AGAS_TEXT_DOMAIN),
                array('status' => 403)
            );
        }
        
        // リクエストからパラメータを取得
        $params = $request->get_params();
        
        // 必須パラメータのチェック
        if (!isset($params['title']) || empty($params['title'])) {
            return new WP_Error('empty_title', __('タイトルは必須です。', AGAS_TEXT_DOMAIN), array('status' => 400));
        }
        
        // パラメータの検証と型変換
        $title = sanitize_text_field($params['title']);
        if (strlen($title) > 200) { // 最大長を制限
            return new WP_Error('title_too_long', __('タイトルは200文字以内で入力してください。', AGAS_TEXT_DOMAIN), array('status' => 400));
        }
        
        $keywords = isset($params['keywords']) ? sanitize_text_field($params['keywords']) : '';
        
        // 選択肢の検証
        $allowed_types = array('blog', 'news', 'review', 'howto', 'listicle');
        $type = isset($params['type']) && in_array($params['type'], $allowed_types) 
            ? $params['type'] 
            : 'blog';
        
        $allowed_lengths = array('short', 'medium', 'long');
        $length = isset($params['length']) && in_array($params['length'], $allowed_lengths) 
            ? $params['length'] 
            : 'medium';
        
        $allowed_tones = array('casual', 'professional', 'friendly', 'academic', 'funny');
        $tone = isset($params['tone']) && in_array($params['tone'], $allowed_tones) 
            ? $params['tone'] 
            : 'professional';
        
        $additional_instructions = isset($params['additionalInstructions']) 
            ? wp_kses_post(trim($params['additionalInstructions'])) 
            : '';
        
        // ブール値の型変換
        $auto_headings = isset($params['autoHeadings']) && $params['autoHeadings'] === true;
        $add_conclusion = isset($params['addConclusion']) && $params['addConclusion'] === true;
        $include_faq = isset($params['includeFaq']) && $params['includeFaq'] === true;
        
        // ジェネレーターを初期化してコンテンツを生成
        $generator = new AnimaGate_AI_Suite_Generator($this->plugin_options);
        $content = $generator->generate_content_with_openai(
            $title,
            $keywords,
            $type,
            $length,
            $tone,
            $additional_instructions,
            $auto_headings,
            $add_conclusion,
            $include_faq
        );
        
        if (is_wp_error($content)) {
            return new WP_Error(
                'generation_error',
                $content->get_error_message(),
                array('status' => 500)
            );
        }
        
        // 生成されたHTMLを安全な形式でサニタイズ
        $content = wp_kses_post($content);
        
        // HTML要素をクリーンアップ
        $content = $this->cleanup_html_content($content);
        
        return array(
            'success' => true,
            'content' => $content
        );
    }

    /**
     * 抜粋生成エンドポイント
     */
    public function generate_excerpt_endpoint($request) {
        // ノンスチェック
        $nonce = $request->get_header('X-WP-Nonce');
        if (!wp_verify_nonce($nonce, 'wp_rest')) {
            return new WP_Error(
                'rest_forbidden',
                __('セキュリティチェックに失敗しました。ページを更新してからもう一度お試しください。', AGAS_TEXT_DOMAIN),
                array('status' => 403)
            );
        }
        
        // リクエストからパラメータを取得
        $params = $request->get_params();
        
        // 必須パラメータのチェック
        if (!isset($params['content']) || empty($params['content'])) {
            return new WP_Error('empty_content', __('抜粋を作成するためのコンテンツが必要です。', AGAS_TEXT_DOMAIN), array('status' => 400));
        }
        
        // コンテンツのサニタイズ
        $content = wp_kses_post($params['content']);
        
        // 抜粋のタイプ（通常/短い/長い）
        $tone = isset($params['tone']) && in_array($params['tone'], array('casual', 'professional', 'friendly', 'academic')) 
            ? $params['tone'] 
            : 'professional';
        
        // 抜粋の文字数（デフォルトは100文字）
        $length = isset($params['length']) && is_numeric($params['length']) && $params['length'] >= 50 && $params['length'] <= 500
            ? intval($params['length'])
            : 100;
        
        // ジェネレーターを初期化して抜粋を生成
        $generator = new AnimaGate_AI_Suite_Generator($this->plugin_options);
        $excerpt = $generator->generate_excerpt_with_openai($content, $tone, $length);
        
        if (is_wp_error($excerpt)) {
            return new WP_Error(
                'generation_error',
                $excerpt->get_error_message(),
                array('status' => 500)
            );
        }
        
        // 生成された抜粋をサニタイズ
        $excerpt = sanitize_text_field($excerpt);
        
        return array(
            'success' => true,
            'excerpt' => $excerpt
        );
    }

    /**
     * ブロックエディターにモーダルコンテンツの拡張ポイントを提供
     * 
     * @param array $modal_content モーダルコンテンツの設定
     * @return array フィルター適用後のモーダルコンテンツ
     */
    public function filter_modal_content($modal_content) {
        return apply_filters('animagate_ai_suite_modal_content', $modal_content);
    }

    /**
     * HTML要素をクリーンアップするヘルパーメソッド
     * 
     * @param string $html HTML文字列
     * @return string クリーンアップされたHTML
     */
    private function cleanup_html_content($html) {
        // 入力値をサニタイズ
        $html = wp_kses_post($html);
        
        // <!DOCTYPE>タグを削除
        $html = preg_replace('/<!DOCTYPE[^>]*>/i', '', $html);
        
        // headタグとその中身を削除
        $html = preg_replace('/<head>.*?<\/head>/is', '', $html);
        
        // html, body, meta, titleタグを削除
        $html = preg_replace('/<html[^>]*>|<\/html>|<body[^>]*>|<\/body>|<meta[^>]*>|<title>.*?<\/title>/is', '', $html);
        
        // スクリプトタグを削除 (XSS対策)
        $html = preg_replace('/<script\b[^>]*>(.*?)<\/script>/is', '', $html);
        
        // インラインイベントハンドラを削除 (XSS対策)
        $html = preg_replace('/\s+on\w+\s*=\s*(["\']).*?\1/i', '', $html);
        
        // データURLスキームを削除 (XSS対策)
        $html = preg_replace('/\s+href\s*=\s*(["\'])javascript:.*?\1/i', ' href="#"', $html);
        $html = preg_replace('/\s+src\s*=\s*(["\'])data:.*?\1/i', '', $html);
        
        // 余分な空白行を削除
        $html = preg_replace('/^\s*[\r\n]+/m', '', $html);
        
        return trim($html);
    }
}