<?php
/**
 * AI記事生成機能を実装するクラス
 */
class AnimaGate_AI_Suite_Generator {
    private $plugin_options;
    private $openai_api_key;

    public function __construct($plugin_options) {
        $this->plugin_options = $plugin_options;
        $this->openai_api_key = $this->get_api_key();
    }

    /**
     * 初期化
     */
    public function init() {
        // AJAXハンドラーを登録
        add_action('wp_ajax_generate_ai_content', array($this, 'ajax_generate_content'));
    }

    /**
     * APIキー取得ロジック改善
     */
    private function get_api_key() {
        // 環境変数から取得を試みる（最も安全な方法）
        if (defined('ANIMAGATE_OPENAI_API_KEY') && !empty(ANIMAGATE_OPENAI_API_KEY)) {
            return ANIMAGATE_OPENAI_API_KEY;
        }
        
        // 設定から取得（代替手段）
        return isset($this->plugin_options['api_key']) ? $this->plugin_options['api_key'] : '';
    }

    /**
     * 記事を生成するAJAXハンドラー
     */
    public function ajax_generate_content() {
        // ノンス検証
        if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'animagate_ai_suite_nonce')) {
            wp_send_json_error(array('message' => __('セキュリティチェックに失敗しました。', AGAS_TEXT_DOMAIN)));
            exit;
        }
        
        // アクセス権限チェック
        if (!current_user_can('edit_posts')) {
            wp_send_json_error(array('message' => __('この操作を行う権限がありません。', AGAS_TEXT_DOMAIN)));
            exit;
        }
        
        // パラメータの取得と厳格なサニタイズ
        $title = isset($_POST['title']) ? sanitize_text_field($_POST['title']) : '';
        $keywords = isset($_POST['keywords']) ? sanitize_text_field($_POST['keywords']) : '';
        $type = isset($_POST['type']) ? sanitize_text_field($_POST['type']) : 'blog';
        $length = isset($_POST['length']) ? sanitize_text_field($_POST['length']) : 'medium';
        $tone = isset($_POST['tone']) ? sanitize_text_field($_POST['tone']) : 'professional';
        $additional_instructions = isset($_POST['additional_instructions']) ? sanitize_textarea_field($_POST['additional_instructions']) : '';
        
        // 選択肢の検証
        $allowed_types = array('blog', 'news', 'review', 'howto', 'listicle');
        if (!in_array($type, $allowed_types)) {
            $type = 'blog';
        }
        
        $allowed_lengths = array('short', 'medium', 'long');
        if (!in_array($length, $allowed_lengths)) {
            $length = 'medium';
        }
        
        $allowed_tones = array('casual', 'professional', 'friendly', 'academic', 'funny');
        if (!in_array($tone, $allowed_tones)) {
            $tone = 'professional';
        }
        
        // オプション - 安全にブール値に変換
        $auto_headings = isset($_POST['auto_headings']) && ($_POST['auto_headings'] === '1' || $_POST['auto_headings'] === 1 || $_POST['auto_headings'] === true);
        $add_conclusion = isset($_POST['add_conclusion']) && ($_POST['add_conclusion'] === '1' || $_POST['add_conclusion'] === 1 || $_POST['add_conclusion'] === true);
        $include_faq = isset($_POST['include_faq']) && ($_POST['include_faq'] === '1' || $_POST['include_faq'] === 1 || $_POST['include_faq'] === true);
        
        if (empty($title)) {
            wp_send_json_error(array('message' => __('タイトルは必須です。', AGAS_TEXT_DOMAIN)));
            exit;
        }
        
        // OpenAI APIを使用して記事を生成
        $content = $this->generate_content_with_openai(
            $title,
            $keywords,
            $type,
            $length,
            $tone,
            $additional_instructions,
            $auto_headings,
            $add_conclusion,
            $include_faq
        );
        
        if (is_wp_error($content)) {
            wp_send_json_error(array('message' => $content->get_error_message()));
            exit;
        }
        
        // HTML要素をクリーンアップ
        $content = $this->cleanup_html_content($content);
        
        // HTMLをブロックエディタ形式に変換
        $content = $this->convert_to_block_format($content);
        
        // 記事を作成
        $post_data = array(
            'post_title'    => $title,
            'post_content'  => $content,
            'post_status'   => 'draft',
            'post_author'   => get_current_user_id(),
            'post_type'     => 'post',
        );
        
        // 投稿を挿入
        $post_id = wp_insert_post($post_data, true);
        
        if (is_wp_error($post_id)) {
            wp_send_json_error(array('message' => $post_id->get_error_message()));
            exit;
        }
        
        // 生成フラグを設定
        update_post_meta($post_id, '_ai_generated_content', '1');
        
        // キーワードをタグとして追加
        if (!empty($keywords)) {
            $keyword_array = array_map('trim', explode(',', $keywords));
            wp_set_post_tags($post_id, $keyword_array);
        }
        
        // 成功レスポンス
        wp_send_json_success(array(
            'message' => __('記事が正常に生成されました。', AGAS_TEXT_DOMAIN),
            'post_id' => $post_id,
            'edit_url' => get_edit_post_link($post_id, '')
        ));
    }

    /**
     * HTML要素をクリーンアップするヘルパーメソッド
     * 
     * @param string $html HTML文字列
     * @return string クリーンアップされたHTML
     */
    private function cleanup_html_content($html) {
        // 入力値をサニタイズ
        $html = wp_kses_post($html);
        
        // <!DOCTYPE>タグを削除
        $html = preg_replace('/<!DOCTYPE[^>]*>/i', '', $html);
        
        // headタグとその中身を削除
        $html = preg_replace('/<head>.*?<\/head>/is', '', $html);
        
        // html, body, meta, titleタグを削除
        $html = preg_replace('/<html[^>]*>|<\/html>|<body[^>]*>|<\/body>|<meta[^>]*>|<title>.*?<\/title>/is', '', $html);
        
        // スクリプトタグを削除 (XSS対策)
        $html = preg_replace('/<script\b[^>]*>(.*?)<\/script>/is', '', $html);
        
        // インラインイベントハンドラを削除 (XSS対策)
        $html = preg_replace('/\s+on\w+\s*=\s*(["\']).*?\1/i', '', $html);
        
        // データURLスキームを削除 (XSS対策)
        $html = preg_replace('/\s+href\s*=\s*(["\'])javascript:.*?\1/i', ' href="#"', $html);
        $html = preg_replace('/\s+src\s*=\s*(["\'])data:.*?\1/i', '', $html);
        
        // 余分な空白行を削除
        $html = preg_replace('/^\s*[\r\n]+/m', '', $html);
        
        return trim($html);
    }

    /**
     * HTMLコンテンツをブロックエディタ形式に変換
     * 
     * @param string $html 変換するHTML
     * @return string ブロックエディタ形式のコンテンツ
     */
    private function convert_to_block_format($html) {
        // 入力HTMLをサニタイズ
        $html = wp_kses_post($html);
        
        // 単純なパターンマッチングによる変換
        $content = '';
        
        // 簡易的なHTMLパーサー
        $html = trim($html);
        
        // 段落を分割
        $pattern = '/<(h[1-6]|p|ul|ol|blockquote|pre)([^>]*)>(.*?)<\/\1>/is';
        preg_match_all($pattern, $html, $matches, PREG_SET_ORDER);
        
        foreach ($matches as $match) {
            $tag = $match[1];
            $attributes = $match[2];
            // コンテンツ内部も再度サニタイズ
            $content_inside = wp_kses_post($match[3]);
            
            switch ($tag) {
                case 'h1':
                    $content .= "<!-- wp:heading {\"level\":1} -->\n<h1 class=\"wp-block-heading\">" . $content_inside . "</h1>\n<!-- /wp:heading -->\n\n";
                    break;
                    
                case 'h2':
                    $content .= "<!-- wp:heading -->\n<h2 class=\"wp-block-heading\">" . $content_inside . "</h2>\n<!-- /wp:heading -->\n\n";
                    break;
                    
                case 'h3':
                    $content .= "<!-- wp:heading {\"level\":3} -->\n<h3 class=\"wp-block-heading\">" . $content_inside . "</h3>\n<!-- /wp:heading -->\n\n";
                    break;
                    
                case 'h4':
                    $content .= "<!-- wp:heading {\"level\":4} -->\n<h4 class=\"wp-block-heading\">" . $content_inside . "</h4>\n<!-- /wp:heading -->\n\n";
                    break;
                    
                case 'h5':
                    $content .= "<!-- wp:heading {\"level\":5} -->\n<h5 class=\"wp-block-heading\">" . $content_inside . "</h5>\n<!-- /wp:heading -->\n\n";
                    break;
                    
                case 'h6':
                    $content .= "<!-- wp:heading {\"level\":6} -->\n<h6 class=\"wp-block-heading\">" . $content_inside . "</h6>\n<!-- /wp:heading -->\n\n";
                    break;
                    
                case 'p':
                    $content .= "<!-- wp:paragraph -->\n<p>" . $content_inside . "</p>\n<!-- /wp:paragraph -->\n\n";
                    break;
                    
                case 'ul':
                    // リスト項目を処理
                    $list_items = '';
                    $li_pattern = '/<li>(.*?)<\/li>/is';
                    preg_match_all($li_pattern, $content_inside, $li_matches);
                    
                    if (!empty($li_matches[0])) {
                        foreach ($li_matches[1] as $li_content) {
                            // リスト項目もサニタイズ
                            $li_content = wp_kses_post($li_content);
                            $list_items .= "<!-- wp:list-item -->\n<li>" . $li_content . "</li>\n<!-- /wp:list-item -->\n";
                        }
                        $content .= "<!-- wp:list -->\n<ul class=\"wp-block-list\">" . $list_items . "</ul>\n<!-- /wp:list -->\n\n";
                    }
                    break;
                    
                case 'ol':
                    // 番号付きリスト項目を処理
                    $list_items = '';
                    $li_pattern = '/<li>(.*?)<\/li>/is';
                    preg_match_all($li_pattern, $content_inside, $li_matches);
                    
                    if (!empty($li_matches[0])) {
                        foreach ($li_matches[1] as $li_content) {
                            // リスト項目もサニタイズ
                            $li_content = wp_kses_post($li_content);
                            $list_items .= "<!-- wp:list-item -->\n<li>" . $li_content . "</li>\n<!-- /wp:list-item -->\n";
                        }
                        $content .= "<!-- wp:list {\"ordered\":true} -->\n<ol class=\"wp-block-list\">" . $list_items . "</ol>\n<!-- /wp:list -->\n\n";
                    }
                    break;
                    
                case 'blockquote':
                    $content .= "<!-- wp:quote -->\n<blockquote class=\"wp-block-quote\"><p>" . $content_inside . "</p></blockquote>\n<!-- /wp:quote -->\n\n";
                    break;
                    
                case 'pre':
                    $content .= "<!-- wp:preformatted -->\n<pre class=\"wp-block-preformatted\">" . $content_inside . "</pre>\n<!-- /wp:preformatted -->\n\n";
                    break;
            }
        }
        
        // 変換できなかった場合、元のHTMLを返す（ただしサニタイズされた状態で）
        if (empty($content)) {
            return wp_kses_post($html);
        }
        
        return $content;
    }
    
    /**
     * ノードのinner HTMLを取得して安全な形式で返す
     * 
     * @param DOMNode $node 対象のノード
     * @return string 安全なinnerHTML
     */
    private function get_node_inner_html($node) {
        $innerHTML = '';
        $children = $node->childNodes;
        
        foreach ($children as $child) {
            $rawHTML = $node->ownerDocument->saveHTML($child);
            // 取得したHTMLをサニタイズして安全にする
            $innerHTML .= wp_kses_post($rawHTML);
        }
        
        return $innerHTML;
    }

    /**
     * OpenAI APIを使用して記事を生成
     *
     * @param string $title タイトル
     * @param string $keywords キーワード
     * @param string $type 記事タイプ
     * @param string $length 記事の長さ
     * @param string $tone 記事のトーン
     * @param string $additional_instructions 追加指示
     * @param bool $auto_headings 見出しを自動生成
     * @param bool $add_conclusion 結論を追加
     * @param bool $include_faq FAQを含める
     * @return string|WP_Error 生成された記事の内容、またはエラー
     */
    public function generate_content_with_openai($title, $keywords, $type, $length, $tone, $additional_instructions, $auto_headings, $add_conclusion, $include_faq) {
        // 入力値の厳格なサニタイズ
        $title = sanitize_text_field($title);
        $keywords = sanitize_text_field($keywords);
        $type = sanitize_text_field($type);
        $length = sanitize_text_field($length);
        $tone = sanitize_text_field($tone);
        $additional_instructions = wp_kses_post($additional_instructions); // HTMLを許可するが安全な形式に

        if (empty($this->openai_api_key)) {
            return new WP_Error('no_api_key', __('OpenAI APIキーが設定されていません。', AGAS_TEXT_DOMAIN));
        }
        
        // 長さの設定
        $content_length = '';
        switch ($length) {
            case 'short':
                $content_length = '約500字';
                break;
            case 'medium':
                $content_length = '約1000字';
                break;
            case 'long':
                $content_length = '約2000字';
                break;
            default:
                $content_length = '約1000字'; // デフォルト値を設定
        }
        
        // トーンの設定
        $content_tone = '';
        switch ($tone) {
            case 'casual':
                $content_tone = 'カジュアルで親しみやすい';
                break;
            case 'professional':
                $content_tone = 'プロフェッショナルでビジネスライク';
                break;
            case 'friendly':
                $content_tone = 'フレンドリーで共感的';
                break;
            case 'academic':
                $content_tone = 'アカデミックで論理的';
                break;
            case 'funny':
                $content_tone = 'ユーモアを交えた面白い';
                break;
            default:
                $content_tone = 'プロフェッショナルでビジネスライク'; // デフォルト値を設定
        }
        
        // 記事タイプの設定
        $content_type = '';
        switch ($type) {
            case 'blog':
                $content_type = 'ブログ記事';
                break;
            case 'news':
                $content_type = 'ニュース記事';
                break;
            case 'review':
                $content_type = 'レビュー記事';
                break;
            case 'howto':
                $content_type = 'ハウツー記事';
                break;
            case 'listicle':
                $content_type = 'リスト形式の記事';
                break;
            default:
                $content_type = 'ブログ記事'; // デフォルト値を設定
        }
        
        // プロンプトの作成
        $system_prompt = "あなたはプロのWordPressブログライターです。WordPressのブロックエディタで編集しやすい記事を作成します。";
        
        $user_prompt = "以下の条件で{$content_type}を執筆してください。\n
        タイトル: {$title}\n";
        
        if (!empty($keywords)) {
            $user_prompt .= "キーワード: {$keywords}\n";
        }
        
        $user_prompt .= "文字数: {$content_length}\n
        トーン: {$content_tone}\n";
        
        // オプション設定をプロンプトに追加
        $options = array();
        if ($auto_headings) {
            $options[] = "記事には適切な見出し（h2, h3）を含めてください";
        }
        if ($add_conclusion) {
            $options[] = "記事の最後にまとめや結論を追加してください";
        }
        if ($include_faq) {
            $options[] = "記事の最後にFAQセクションを追加してください（3〜5つの質問と回答）";
        }
        
        if (!empty($options)) {
            $user_prompt .= "追加要件:\n- " . implode("\n- ", $options) . "\n";
        }
        
        if (!empty($additional_instructions)) {
            $user_prompt .= "\n追加指示:\n{$additional_instructions}\n";
        }
        
        $user_prompt .= "\n出力形式: WordPressブログ記事に適したHTML形式で出力してください。見出しにはh2、h3タグ、段落にはpタグ、リストにはul、liタグなどを使用してください。meta、title、html、head、bodyタグは含めないでください。bodyタグ内のHTMLコンテンツのみを生成してください。";
        
        // OpenAI APIリクエストデータの準備
        $max_tokens = isset($this->plugin_options['max_tokens']) ? intval($this->plugin_options['max_tokens']) : 1500;
        $temperature = isset($this->plugin_options['temperature']) ? floatval($this->plugin_options['temperature']) : 0.7;
        $model = isset($this->plugin_options['model']) ? sanitize_text_field($this->plugin_options['model']) : 'gpt-3.5-turbo';
        
        // 値の範囲チェック
        if ($max_tokens < 100) {
            $max_tokens = 100;
        } elseif ($max_tokens > 4000) {
            $max_tokens = 4000;
        }
        
        if ($temperature < 0) {
            $temperature = 0;
        } elseif ($temperature > 1) {
            $temperature = 1;
        }
        
        $request_data = array(
            'model' => $model,
            'messages' => array(
                array(
                    'role' => 'system',
                    'content' => $system_prompt
                ),
                array(
                    'role' => 'user',
                    'content' => $user_prompt
                )
            ),
            'max_tokens' => $max_tokens,
            'temperature' => $temperature,
        );
        
        // APIリクエストの設定
        $args = array(
            'body' => json_encode($request_data),
            'headers' => array(
                'Content-Type' => 'application/json',
                'Authorization' => 'Bearer ' . $this->openai_api_key
            ),
            'timeout' => 60
        );
        
        // APIリクエストの送信
        $response = wp_remote_post('https://api.openai.com/v1/chat/completions', $args);
        
        // レスポンスのチェック
        if (is_wp_error($response)) {
            return $response;
        }
// レスポンスヘッダーとステータスコードのチェック
        $status_code = wp_remote_retrieve_response_code($response);
        if ($status_code !== 200) {
            $error_message = wp_remote_retrieve_response_message($response);
            return new WP_Error('api_error', sprintf(__('API リクエストエラー: %s', AGAS_TEXT_DOMAIN), $error_message));
        }
        
        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body, true);
        
        if (!isset($data['choices'][0]['message']['content'])) {
            // APIからのエラーメッセージを公開せず、一般的なエラーメッセージを使用
            $error_code = isset($data['error']['code']) ? sanitize_text_field($data['error']['code']) : 'unknown_error';
            
            // エラーコードに応じた適切なメッセージを返す
            switch ($error_code) {
                case 'invalid_api_key':
                    $error_message = __('APIキーが無効です。設定を確認してください。', AGAS_TEXT_DOMAIN);
                    break;
                case 'insufficient_quota':
                    $error_message = __('APIの利用制限に達しました。', AGAS_TEXT_DOMAIN);
                    break;
                case 'rate_limit_exceeded':
                    $error_message = __('APIリクエストが多すぎます。しばらく待ってから再試行してください。', AGAS_TEXT_DOMAIN);
                    break;
                default:
                    $error_message = __('APIへのリクエスト中にエラーが発生しました。', AGAS_TEXT_DOMAIN);
            }
            
            // 開発者向けにログ記録（本番環境では無効化または適切に制限する）
            if (defined('WP_DEBUG') && WP_DEBUG) {
                error_log('OpenAI API Error: ' . print_r($data['error'], true));
            }
            
            return new WP_Error('api_error', $error_message);
        }
        
        // 生成されたコンテンツを取得し、サニタイズ
        $generated_content = $data['choices'][0]['message']['content'];
        $generated_content = wp_kses_post($generated_content);
        
        return $generated_content;
    }

    /**
     * OpenAI APIを使用して抜粋を生成
     *
     * @param string $content 元のコンテンツ
     * @param string $tone 抜粋のトーン
     * @param int $length 抜粋の長さ（文字数）
     * @return string|WP_Error 生成された抜粋、またはエラー
     */
    public function generate_excerpt_with_openai($content, $tone = 'professional', $length = 100) {
        // 入力値の厳格なサニタイズ
        $content = wp_kses_post($content);
        $tone = sanitize_text_field($tone);
        $length = intval($length);

        // 値の範囲チェック
        if ($length < 50) {
            $length = 50;
        } elseif ($length > 500) {
            $length = 500;
        }

        if (empty($this->openai_api_key)) {
            return new WP_Error('no_api_key', __('OpenAI APIキーが設定されていません。', AGAS_TEXT_DOMAIN));
        }
        
        // 抜粋のトーン設定
        $excerpt_tone = '';
        switch ($tone) {
            case 'casual':
                $excerpt_tone = 'カジュアル';
                break;
            case 'professional':
                $excerpt_tone = 'プロフェッショナル';
                break;
            case 'friendly':
                $excerpt_tone = 'フレンドリー';
                break;
            case 'academic':
                $excerpt_tone = 'アカデミック';
                break;
            default:
                $excerpt_tone = 'プロフェッショナル'; // デフォルト値
        }
        
        // プロンプトの作成
        $system_prompt = "あなたは WordPress サイトのコンテンツ編集者です。記事の内容から適切な抜粋を作成します。";
        
        $user_prompt = "以下の記事の内容から、記事の魅力を伝える約{$length}文字の抜粋を{$excerpt_tone}トーンで作成してください。\n\n
        記事内容: " . mb_substr($content, 0, 3000) . "\n\n
        抜粋は読者が記事をクリックしたくなるような魅力的な内容にしてください。
        キーワードや重要なポイントを含め、記事の価値を簡潔に伝えてください。
        HTMLタグは含めず、プレーンテキストで出力してください。
        文字数はできるだけ{$length}文字に近づけてください。";
        
        // OpenAI APIリクエストデータの準備
        $max_tokens = 300;
        $temperature = 0.7;
        $model = isset($this->plugin_options['model']) ? sanitize_text_field($this->plugin_options['model']) : 'gpt-3.5-turbo';
        
        $request_data = array(
            'model' => $model,
            'messages' => array(
                array(
                    'role' => 'system',
                    'content' => $system_prompt
                ),
                array(
                    'role' => 'user',
                    'content' => $user_prompt
                )
            ),
            'max_tokens' => intval($max_tokens),
            'temperature' => floatval($temperature),
        );
        
        // APIリクエストの設定
        $args = array(
            'body' => json_encode($request_data),
            'headers' => array(
                'Content-Type' => 'application/json',
                'Authorization' => 'Bearer ' . $this->openai_api_key
            ),
            'timeout' => 30
        );
        
        // APIリクエストの送信
        $response = wp_remote_post('https://api.openai.com/v1/chat/completions', $args);
        
        // レスポンスのチェック
        if (is_wp_error($response)) {
            return $response;
        }
        
        $status_code = wp_remote_retrieve_response_code($response);
        if ($status_code !== 200) {
            $error_message = wp_remote_retrieve_response_message($response);
            return new WP_Error('api_error', sprintf(__('API リクエストエラー: %s', AGAS_TEXT_DOMAIN), $error_message));
        }
        
        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body, true);
        
        if (!isset($data['choices'][0]['message']['content'])) {
            // APIからのエラーメッセージを安全に処理
            $error_code = isset($data['error']['code']) ? sanitize_text_field($data['error']['code']) : 'unknown_error';
            
            // エラーコードに応じた適切なメッセージを返す
            switch ($error_code) {
                case 'invalid_api_key':
                    $error_message = __('APIキーが無効です。設定を確認してください。', AGAS_TEXT_DOMAIN);
                    break;
                case 'insufficient_quota':
                    $error_message = __('APIの利用制限に達しました。', AGAS_TEXT_DOMAIN);
                    break;
                case 'rate_limit_exceeded':
                    $error_message = __('APIリクエストが多すぎます。しばらく待ってから再試行してください。', AGAS_TEXT_DOMAIN);
                    break;
                default:
                    $error_message = __('APIへのリクエスト中にエラーが発生しました。', AGAS_TEXT_DOMAIN);
            }
            
            if (defined('WP_DEBUG') && WP_DEBUG) {
                error_log('OpenAI API Error: ' . print_r($data['error'], true));
            }
            
            return new WP_Error('api_error', $error_message);
        }
        
        // 生成された抜粋を取得して安全にサニタイズ
        $generated_excerpt = trim($data['choices'][0]['message']['content']);
        $generated_excerpt = sanitize_text_field($generated_excerpt);
        
        return $generated_excerpt;
    }
}