<?php
/**
 * プロンプトテンプレート機能を実装する拡張クラス
 */
class AnimaGate_AI_Suite_Prompt_Templates {
    public function __construct() {
        $this->init();
    }

    /**
     * 初期化
     */
    public function init() {
        // モーダルコンテンツをフィルターしてテンプレート選択を追加
        add_filter('animagate_ai_suite_modal_content', array($this, 'add_template_selector'));
        
        // REST API エンドポイントを追加
        add_action('animagate_ai_suite_register_rest_routes', array($this, 'register_template_endpoints'));
        
        // スクリプトにテンプレートデータを追加
        add_filter('animagate_ai_suite_editor_data', array($this, 'add_templates_data'));
    }

    /**
     * モーダルコンテンツにテンプレート選択機能を追加
     */
    public function add_template_selector($content) {
        // テンプレート選択UIを追加するためのデータを設定
        $content['template_selector'] = array(
            'enabled' => true,
            'label' => __('テンプレートを選択', AGAS_TEXT_DOMAIN),
            'description' => __('事前に用意されたテンプレートから選択できます', AGAS_TEXT_DOMAIN),
            'position' => 'top' // モーダル内の表示位置
        );
        
        return $content;
    }

    /**
     * テンプレート関連のREST APIエンドポイントを登録
     */
    public function register_template_endpoints() {
        register_rest_route('animagate-ai-suite/v1', '/templates', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_templates_endpoint'),
            'permission_callback' => function() {
                return current_user_can('edit_posts');
            }
        ));
        
        register_rest_route('animagate-ai-suite/v1', '/templates/(?P<id>\d+)', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_template_endpoint'),
            'permission_callback' => function() {
                return current_user_can('edit_posts');
            }
        ));
    }

    /**
     * テンプレート一覧を取得するエンドポイント
     */
    public function get_templates_endpoint() {
        return array(
            'success' => true,
            'templates' => $this->get_templates()
        );
    }

    /**
     * 特定のテンプレートを取得するエンドポイント
     */
    public function get_template_endpoint($request) {
        $template_id = $request['id'];
        $templates = $this->get_templates();
        
        foreach ($templates as $template) {
            if ($template['id'] == $template_id) {
                return array(
                    'success' => true,
                    'template' => $template
                );
            }
        }
        
        return new WP_Error(
            'template_not_found',
            __('テンプレートが見つかりません', AGAS_TEXT_DOMAIN),
            array('status' => 404)
        );
    }

    /**
     * テンプレートデータをスクリプトに追加
     */
    public function add_templates_data($data) {
        $data['templates'] = $this->get_templates();
        $data['strings']['selectTemplate'] = __('テンプレートを選択', AGAS_TEXT_DOMAIN);
        $data['strings']['noTemplate'] = __('テンプレートなし', AGAS_TEXT_DOMAIN);
        
        return $data;
    }

    /**
     * 利用可能なテンプレート一覧を取得
     */
    private function get_templates() {
        // 実際のプロジェクトでは、データベースからテンプレートを取得するかもしれません
        return array(
            array(
                'id' => 1,
                'name' => __('製品レビュー', AGAS_TEXT_DOMAIN),
                'description' => __('製品の詳細なレビュー記事', AGAS_TEXT_DOMAIN),
                'type' => 'review',
                'additionalInstructions' => __('製品の長所と短所を詳しく説明し、最後に総合評価を追加してください。', AGAS_TEXT_DOMAIN),
                'autoHeadings' => true,
                'addConclusion' => true,
                'includeFaq' => true
            ),
            array(
                'id' => 2,
                'name' => __('ハウツーガイド', AGAS_TEXT_DOMAIN),
                'description' => __('ステップバイステップのガイド記事', AGAS_TEXT_DOMAIN),
                'type' => 'howto',
                'additionalInstructions' => __('各ステップを明確に説明し、必要なツールや材料のリストも含めてください。', AGAS_TEXT_DOMAIN),
                'autoHeadings' => true,
                'addConclusion' => true,
                'includeFaq' => true
            ),
            array(
                'id' => 3,
                'name' => __('ニュース記事', AGAS_TEXT_DOMAIN),
                'description' => __('最新のニュースを伝える記事', AGAS_TEXT_DOMAIN),
                'type' => 'news',
                'additionalInstructions' => __('5W1H（いつ、どこで、誰が、何を、なぜ、どのように）の要素を含めてください。', AGAS_TEXT_DOMAIN),
                'autoHeadings' => true,
                'addConclusion' => false,
                'includeFaq' => false
            ),
            array(
                'id' => 4,
                'name' => __('比較記事', AGAS_TEXT_DOMAIN),
                'description' => __('複数の製品やサービスを比較する記事', AGAS_TEXT_DOMAIN),
                'type' => 'blog',
                'additionalInstructions' => __('比較表や評価基準を含め、それぞれの長所と短所を明確に示してください。最後にベストな選択肢についての提案を追加してください。', AGAS_TEXT_DOMAIN),
                'autoHeadings' => true,
                'addConclusion' => true,
                'includeFaq' => true
            ),
            array(
                'id' => 5,
                'name' => __('トップ10リスト', AGAS_TEXT_DOMAIN),
                'description' => __('特定のカテゴリの上位10項目を紹介する記事', AGAS_TEXT_DOMAIN),
                'type' => 'listicle',
                'additionalInstructions' => __('各項目の特徴や選定理由を詳しく説明し、項目ごとに見出しを使用してください。', AGAS_TEXT_DOMAIN),
                'autoHeadings' => true,
                'addConclusion' => true,
                'includeFaq' => false
            )
        );
    }
}